/**
 *
 * Magnolia Criteria API (http://www.openmindlab.com/lab/products/mgnlcriteria.html)
 * Copyright (C)2009 - 2010, Openmind S.r.l. http://www.openmindonline.it
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.sourceforge.openutils.mgnlcriteria.jcr.query.xpath.impl;

import info.magnolia.cms.core.Content;
import info.magnolia.cms.core.HierarchyManager;
import info.magnolia.cms.core.search.Query;
import info.magnolia.cms.core.search.QueryManager;
import info.magnolia.cms.core.search.QueryResult;
import info.magnolia.cms.util.DeprecationUtil;
import info.magnolia.content2bean.Content2BeanException;
import info.magnolia.content2bean.Content2BeanUtil;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.jcr.RepositoryException;
import javax.jcr.query.InvalidQueryException;

import net.sourceforge.openutils.mgnlcriteria.jcr.query.JCRQueryException;


/**
 * Magnolia XPATH implementation of the <tt>Criteria</tt> interface
 * @author Federico Grilli
 * @version $Id: MagnoliaCriteriaImpl.java 2923 2010-08-29 08:58:29Z fgiust $
 */
@Deprecated
public class MagnoliaCriteriaImpl extends AbstractMagnoliaCriteriaImpl
{

    protected static final String EXCEPTION_MSG = "An error occurred while executing a query. Xpath query was {}. Exception message is {}";

    @Deprecated
    public MagnoliaCriteriaImpl(String path, QueryManager queryManager, String itemType)
    {
        super(path, queryManager, itemType, null);
    }

    @Deprecated
    public MagnoliaCriteriaImpl(String path, QueryManager queryManager, String itemType, Class< ? > classType)
    {
        super(path, queryManager, itemType, classType);
    }

    /**
     * @param hm
     */
    public MagnoliaCriteriaImpl(HierarchyManager hm)
    {
        super(hm);
    }

    @SuppressWarnings("unchecked")
    @Deprecated
    public Collection< ? > list() throws JCRQueryException
    {
        DeprecationUtil.isDeprecated("please use execute() instead");

        if (queryManager == null)
        {
            throw new IllegalStateException("QueryManager cannot be null");
        }

        log.debug("creating statement...");
        long start = System.currentTimeMillis();

        String stmt = toXpathExpression();

        long stop = System.currentTimeMillis();
        log.debug("statement created in {} ms", stop - start);
        Collection< ? > retVal;
        try
        {

            Query q = queryManager.createQuery(stmt, Query.XPATH);

            log.debug("executing Query {}", q.getStatement());
            start = System.currentTimeMillis();

            QueryResult r = q.execute();

            stop = System.currentTimeMillis();
            log.debug("Query executed in {} ms", stop - start);

            log.debug("getting content...");
            start = System.currentTimeMillis();

            retVal = r.getContent(itemType);

            stop = System.currentTimeMillis();

            if (Content.class.isAssignableFrom(classType))
            {
                log.debug("returning a Collection holding objects of type {} ", Content.class.getName());
                retVal = retVal == null ? new ArrayList<Content>() : retVal;
                log.debug("got {} node(s) in {} ms", new Object[]{retVal.size(), stop - start });

                return retVal;

            }
            else
            {
                log.debug("returning a Collection holding objects of type {} ", classType.getName());
                List list = new ArrayList();
                for (Iterator iter = retVal.iterator(); iter.hasNext();)
                {
                    Content node = (Content) iter.next();
                    try
                    {
                        list.add(classType.cast(Content2BeanUtil.toBean(node, true, classType)));
                    }
                    catch (Content2BeanException e)
                    {
                        log.error(e.getMessage());
                        throw new JCRQueryException(stmt, e);
                    }
                }
                log.debug("got {} node(s) in {} ms", new Object[]{list.size(), stop - start });
                return list;
            }

        }
        catch (InvalidQueryException e)
        {
            log.error(EXCEPTION_MSG, new Object[]{stmt, e.getMessage() });
            throw new JCRQueryException(stmt, e);
        }
        catch (RepositoryException e)
        {
            log.error(EXCEPTION_MSG, new Object[]{stmt, e.getMessage() });
            throw new JCRQueryException(stmt, e);
        }
    }

}